﻿using System;
using System.IO.Ports;		//SerialPort
using System.Globalization;	//NumberStyles
using System.Threading;		//Thread
using System.Text;			//Encoding

class	CdcIO	:	SerialPort
{
	public bool cmdError = false;
	
	//コンストラクタ
	//デフォルト設定：COM1, 9600bps, Parity.NONE, Data=8bit, StopBits.One
	public CdcIO() : base()
	{ }
	
	//ポートをオープンする
	//引数	portName: "COMx"（xは数字）
	public bool Open(string portName)
	{
		PortName = portName;
		try
		{
			base.Open();
		}
		catch { return false; }
		return IsOpen;
	}
	
	public int Get(SFR addr)
	{
		Byte n = 0;
		cmdError = false;
		try
		{
			WriteLine(String.Format("{0:x2} ?", (int)addr));
			n = Byte.Parse(ReadLine(), NumberStyles.HexNumber);
		}
		catch { cmdError = true; }
		return (int)n;
	}
	
	public bool Set(SFR addr, int data)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} =", data & 0xFF, (int)addr));
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	
	public bool SetXor(SFR addr, int data)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} ^", data & 0xFF, (int)addr));
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	
	public string Who()
	{
		string s = null;
		int currentReadTimeoutValue = ReadTimeout;
		ReadTimeout = 2000;
		try
		{
			WriteLine("@");
			s = ReadLine();
		}
		catch { }	//タイムアウト発生はキャッチされる
		ReadTimeout = currentReadTimeoutValue;
		return s;
	}
	
	public bool SetAnd(SFR addr, int data)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} &", data & 0xFF, (int)addr));
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	
	public bool SetOr(SFR addr, int data)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} |", data & 0xFF, (int)addr));
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	
    public bool Set2(SFR addr, int data1, int data2)
    {
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} {2:x2} $", data2 & 0xFF, data1 & 0xFF, (int)addr));
			Thread.Sleep(3);
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	
	//LCDを操作する
	public bool LcdCmd(ELCmd cmd) { return LcdCmd(cmd, 0, 0); }
	public bool LcdCmd(ELCmd cmd, int data) { return LcdCmd(cmd, data, 0); }
	public bool LcdCmd(ELCmd cmd, int data1, int data2)
	{
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} {2:x2} #", data2 & 0xFF, data1 & 0xFF, (int)cmd));
			ReadLine();
		}
		catch { return false; }
		return true;
	}
	
	public bool LcdCls()
	{
		return LcdCmd(ELCmd.LCMD_CMD, 0x01);	//LCD_CMD_CLEAR_DISPLAY 0b00000001
	}
	
	public bool LcdPos(int x, int y)
	{
		return LcdCmd(ELCmd.LCMD_CMD, 0x80 | (y * 0x40 + x));	//LCD_CMD_SET_DDRAM_ADRS 0b10000000
	}

	//文字列を表示する
	//・2文字ずつ転送するのでLCMD_CHAR（1文字ずつ）使用時の半分の転送回数で済み、処理時間が短くなる。
	//・AVR-CDC内で文字をバッファに溜めてから表示するので、1文字ずつ転送したときの「ダラダラ表示される」感がない。
	private Encoding sjis = Encoding.GetEncoding("shift_jis");
	private const int BUF_SIZE = 16;	//AVR-CDC内にあるLCD表示用バッファのサイズ
	public bool LcdString(string s)
	{
		if (s.Length % 2 == 1) s += (char)0;	//数合わせにヌル文字を追加する
		byte[] chars = sjis.GetBytes(s);
		int size = Math.Min(BUF_SIZE, chars.Length);
		LcdCmd(ELCmd.LCMD_BUF_READY);
		for (int i = 0; i < size; i += 2)
			LcdCmd(ELCmd.LCMD_BUF_SET, chars[i], chars[i + 1]);
		return LcdCmd(ELCmd.LCMD_STRING);
	}

	//UInt16型またはUInt8型の整数を表示する
	public bool LcdUInt(int n)
	{
		return LcdCmd(ELCmd.LCMD_UINT, ((UInt16)n >> 8) & 0xFF, (UInt16)n & 0xFF);
	}
	
	//外字登録
	public bool LcdDefChar(int charCode, byte[] data)
	{
		if (charCode < 0 || 8 <= charCode) return false;
		if (data == null || data.Length < 8) return false;
		LcdCmd(ELCmd.LCMD_BUF_READY);
		for (int i = 0; i < 8; i += 2)
			LcdCmd(ELCmd.LCMD_BUF_SET, data[i], data[i + 1]);
		return LcdCmd(ELCmd.LCMD_DEFCHAR, charCode);
	}
	
	//EEPROMを操作する
	public int EepromReadByte(int addr)
	{
		return EepromCmd(EERCmd.ERCMD_READ_BYTE, (addr >> 8) & 0xFF, addr & 0xFF);
	}
	
	public int EepromReadWord(int addr)
	{
		return EepromCmd(EERCmd.ERCMD_READ_WORD, (addr >> 8) & 0xFF, addr & 0xFF);
	}
	
	public bool EepromWriteByte(int addr, int data)
	{
		EepromCmd(EERCmd.ERCMD_WRITE_ADDR, (addr >> 8) & 0xFF, addr & 0xFF);
		if (cmdError) return false;
		EepromCmd(EERCmd.ERCMD_WRITE_BYTE, data, 0);
		return !cmdError;
	}

	public bool EepromWriteWord(int addr, int data)
	{
		EepromCmd(EERCmd.ERCMD_WRITE_ADDR, (addr >> 8) & 0xFF, addr & 0xFF);
		if (cmdError) return false;
		EepromCmd(EERCmd.ERCMD_WRITE_WORD, (data >> 8) & 0xFF, data & 0xFF);
		return !cmdError;
	}
	
	private int EepromCmd(EERCmd cmd, int data1, int data2)
	{
		UInt16 n = 0;
		cmdError = false;
		try
		{
			WriteLine(String.Format("{0:x2} {1:x2} {2:x2} :", data2 & 0xFF, data1 & 0xFF, (int)cmd));
			string s = ReadLine().Trim();	//16進数文字列か改行コードのみが返ってくる
			if (0 < s.Length)
				n = UInt16.Parse(s, NumberStyles.HexNumber);
		}
		catch { cmdError = true; }
		return (int)n;
	}

}
