﻿using System;
using System.IO;		//Path
using System.IO.Ports;	//SerialPort
using System.Threading;		//Thread
using System.Diagnostics;	//PerformanceCounter

namespace ConsoleApplication1
{
	class Program
	{
		static int Main(string[] args)
		{
			PerformanceCounter pcCpu = new PerformanceCounter("Processor", "% Processor Time", "_Total", true);
			PerformanceCounter pcFreeMem = new PerformanceCounter("Memory", "Available MBytes", true);
			CdcIO sfr = new CdcIO();
			string portName = null;
			
			Console.Clear();
			
			//引数チェック
			if (args.Length == 1)
			{
				portName = args[0].ToUpper();
			}
			else
			{
				string[] cmdArgs = Environment.GetCommandLineArgs();
				Console.WriteLine("起動方法: " + Path.GetFileName(cmdArgs[0]) + " COMx");
				Console.Write("有効なCOMポート名:");
				foreach (string name in SerialPort.GetPortNames())
					Console.Write(" " + name);
				Console.WriteLine();
				return 0;
			}
			
			//COMポート オープン
			try
			{
				if (!sfr.Open(portName))
				{
					Console.WriteLine(portName + " がオープンできませんでした。");
					return 1;
				}
			}
			catch { return 2; }
			Console.WriteLine(portName + " に接続しました。");
			
			//デバイス確認
			string sWho = sfr.Who();
			if (sWho == null || sWho.Trim() != "cdc-io")	//末尾の\rを削除するためのTrim()
			{
				Console.WriteLine(portName + " にAVR-CDCモジュールがありません。");
				return 3;
			}
			
			//EEPROMから前回の終了日時を読み込む（初回起動時は0xFF）
			int addr = 0;
			int yyyy = sfr.EepromReadWord(addr++);	addr++;
			int mm = sfr.EepromReadByte(addr++);
			int dd = sfr.EepromReadByte(addr++);
			int hh = sfr.EepromReadByte(addr++);
			int ii = sfr.EepromReadByte(addr++);
			int ss = sfr.EepromReadByte(addr++);
			Console.WriteLine(String.Format("前回の終了日時は {0}年{1}月{2}日 {3}時{4:d2}分{5:d2}秒 でした。", yyyy, mm, dd, hh, ii, ss));
			
			//LCD初期化
			sfr.LcdCmd(ELCmd.LCMD_DATA_PORT, (int)SFR.PORTC, (int)SFR.DDRC);
			sfr.LcdCmd(ELCmd.LCMD_DATA_BIT, 0x3C, 0);	//0b00111100, ビットリバースする(1)／しない(0)
			sfr.LcdCmd(ELCmd.LCMD_CTRL_PORT, (int)SFR.PORTC, (int)SFR.DDRC);
			sfr.LcdCmd(ELCmd.LCMD_CTRL_BIT, 0x02, 0x01);	//(E)0b00000010, (RS)0b00000001
			sfr.LcdCmd(ELCmd.LCMD_INIT);
			
			//LCD外字登録（5x8ドット）
			byte[] dots0 = { 0x1B, 0x1B, 0x1B, 0x1B, 0x00, 0x1B, 0x1B, 0x00 };	//「!!」
			sfr.LcdDefChar(0x00, dots0);
			byte[] dots1 = { 0x04, 0x1F, 0x1B, 0x00, 0x0E, 0x04, 0x1F, 0x00 };	//「空」
			sfr.LcdDefChar(0x01, dots1);
			byte[] dots2 = { 0x08, 0x1F, 0x04, 0x1F, 0x02, 0x10, 0x0E, 0x00 };	//「き」
			sfr.LcdDefChar(0x02, dots2);
			
			//リソースモニタ
			//【参考】http://code.msdn.microsoft.com/C-CPU-e79d9255
			Console.WriteLine("リソースモニタ動作中。");
			Console.WriteLine("[K]キーで カナ/英字 表示切り替え。その他のキーで終了。");
			float fCpu, fFreeMem;
			bool isKana = false;
			while (true)
			{
				//キー入力チェック
				if (Console.KeyAvailable)
				{
					if (Console.ReadKey(true).KeyChar != 'k') break;
					while (Console.KeyAvailable) Console.ReadKey(true);		//連打対策
					isKana = !isKana;
					sfr.LcdCls();
				}
				
				fCpu = pcCpu.NextValue();
				fFreeMem = pcFreeMem.NextValue();
				
				//値は単位込みで左詰め表示する
				sfr.LcdPos(0, 0);
				sfr.LcdString(String.Format("CPU: {0,-6}", fCpu.ToString("f1") + "%"));	//[0.0%  ]-[100.0%]
				sfr.LcdPos(0, 1);
				if (isKana)
					sfr.LcdString(String.Format("{1}{2}ﾒﾓﾘ: {0,-6}", fFreeMem.ToString("f0") + "MB", (char)0x01, (char)0x02));	//[0MB   ]-[9999MB]
				else
					sfr.LcdString(String.Format("FreeMem: {0,-6}", fFreeMem.ToString("f0") + "MB"));	//[0MB   ]-[9999MB]
				
				sfr.LcdPos(15, 0);
				sfr.LcdCmd(ELCmd.LCMD_CHAR, (50.0f <= fCpu) ? (char)0x00 : ' ');	//CPU負荷50％以上で「!!」表示
				
				//Console.WriteLine(String.Format("CPU使用率 {0:f1}%", fCpu));
				//Console.WriteLine(String.Format("空きメモリ {0:f0}MB", fFreeMem));
				Thread.Sleep(1000);
				GC.Collect();
			}
			while (Console.KeyAvailable) Console.ReadKey(true);		//キーバッファを空にする
			
			pcCpu.Dispose();
			pcFreeMem.Dispose();
			
			//EEPROMに今回の終了日時を書き込む
			DateTime dt = DateTime.Now;
			addr = 0;
			sfr.EepromWriteWord(addr++, dt.Year); addr++;
			sfr.EepromWriteByte(addr++, dt.Month);
			sfr.EepromWriteByte(addr++, dt.Day);
			sfr.EepromWriteByte(addr++, dt.Hour);
			sfr.EepromWriteByte(addr++, dt.Minute);
			sfr.EepromWriteByte(addr++, dt.Second);
			
			//LCD画面消去
			sfr.LcdCls();
			
			//使用したレジスタの後始末
			sfr.LcdCmd(ELCmd.LCMD_INIT);
			sfr.Set(SFR.DDRC, 0);
			sfr.Set(SFR.PORTC, 0);
			
			//COMポート クローズ
			try { sfr.Close(); }
			catch { }
			
			Console.WriteLine("リソースモニタ終了。");
			return 0;
		}
		
	}
}
